@@ -109,7 +109,8 @@ class InspectMapper(Container):
         col = mapper.c[field_name] 
         
         try:
-            col_spec = mapper.c[field_name].type.get_col_spec()
+            col_spec = mapper.c[field_name].type.compile(
+                dialect=mapper.local.table.metadata.bind.engine.dialect)
         except (NotImplementedError, AttributeError),e:
             col_spec = None
         d = Container({
@@ -446,6 +447,51 @@ class InspectMapper(Container):
             return issubclass(db_type, (sqltypes.Text))
         return False
 
+    def is_enum(self, field_name):
+        """
+        Return true if this field is  date
+        """
+        from sqlkit import fields
+
+        col = self.fields[field_name]['col']
+        try:
+            field = col.info.get('field', None)
+            if field:
+                return issubclass(field, fields.EnumField)
+            return col.info.get('render') == 'enum'
+        except AttributeError:
+            pass
+
+    def is_image(self, field_name):
+        """
+        Return true if this field is  date
+        """
+        from sqlkit import fields
+
+        col = self.fields[field_name]['col']
+        try:
+            field = col.info.get('field', None)
+            if field:
+                return issubclass(field, fields.ImageField)
+            return col.info.get('render') == 'image'
+        except AttributeError:
+            pass
+
+    def is_file(self, field_name):
+        """
+        Return true if this field is  date
+        """
+        from sqlkit import fields
+
+        col = self.fields[field_name]['col']
+        try:
+            field = col.info.get('field', None)
+            if field:
+                return issubclass(field, fields.FileField)
+            return col.info.get('render') == 'filename'
+        except AttributeError:
+            pass
+
     def is_nullable(self, field_name):
         """
         Return true if this field is  date
@@ -477,6 +523,8 @@ def get_foreign_info(ForeignKey, names=True):
             return (fkcol.column.table, fkcol.column)
     
 
+
+
 def get_props_for_delete_orphan(mapper, field_name):
     """
     discover if field_name setting needs to set an object to fullfill the request
