@@ -23,8 +23,10 @@ mapper. column.set_cell_data_func on each column will set the value from there.
 
 import sys
 import re
+import os
 import gtk
 import weakref
+import shutil
 import datetime
 
 import gobject
@@ -34,7 +36,7 @@ import sqlalchemy
 import sqlkit
 from sqlkit import debug as dbg, _, exc, fields
 from sqlkit.layout import widgets, get_label
-from sqlkit.misc.utils import Container, str2list
+from sqlkit.misc.utils import Container, str2list, open_file
 import utils as tv_utils
 import cell_renderers
 import totals
@@ -45,16 +47,15 @@ from sqlkit.widgets.table.tablewidgets import CellWidget
 
 class SqlTable(sqlwidget.SqlWidget):
     """
-    Main widget to represent data of a sqlalchemy selectable (SELECT of a TABLE, e JOIN,
+    Main widget to represent data of a sqlalchemy selectable (SELECT of a TABLE or JOIN
     & similar).
 
     A SqlTable can have different :ref:`views <views>` of the same data
     represented in different TreeVies.
 
     A SqlTable can be viewed alone or as part of a composite widget, that is
-    normally a mask with :ref:`relationships`. Each SqlTable (as SqlMask has a ``records``
-    attribute that holds the list of records that it represents, while the model has
-    possibly also other objects as totals and headers.
+    normally a mask with :ref:`relationships`: be sure to understand this
+    part as it's probably one of the more powerfull features of Sqlkit.
 
     SqlTables can have both ListStores or TreeStores. The latter allows to represent
     rows in a hierarchy.
@@ -66,7 +67,7 @@ class SqlTable(sqlwidget.SqlWidget):
     __gsignals__ = {
         'button-press-event' : (gobject.SIGNAL_RUN_LAST,
                            gobject.TYPE_BOOLEAN,
-                                # event, obj, field_name, treeview
+                                # event, obj, field_name, treeview, view
                            (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT, gobject.TYPE_STRING,
                             gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT),
                            ),
@@ -77,7 +78,21 @@ class SqlTable(sqlwidget.SqlWidget):
                            ),
         }
 
-    __metaclass__ = dbg.LogTheMethods
+    views = None
+    "A container for the possible :ref:`views`. It always have a default view named ``main``"
+
+    totals = None
+    "class for :ref:`totals`: see the specific documentation for further info"
+    
+    edited_path = None
+    ## For some reason :meth: is not turned in a link...
+    """The path of the edited row (a tuple). It's not the output of
+    :meth:`get_selected_path` since when you change the selected row the
+    validation is triggered on the *previuos* selected_row that is pointed
+    to by ``edited_path``. 
+    """
+
+#    __metaclass__ = dbg.LogTheMethods
     currently_edited_field_name = None
     mapper = None
     cell_entry = None
@@ -238,6 +253,14 @@ class SqlTable(sqlwidget.SqlWidget):
         assert view_name in self.views, "Missing view_name: %s" % view_name
         self.views[view_name].set_field_list(field_list)
 
+    def _get_edited_path(self):
+        "return the curretly edited path as a tuple"
+
+        return self.edited_row_model_path and \
+               tuple(int(x) for x in self.edited_row_model_path.split(':'))
+
+    edited_path = property(_get_edited_path)
+
 ###### Actions/UiManager
     def prepare_actions(self):
         """
@@ -248,7 +271,11 @@ class SqlTable(sqlwidget.SqlWidget):
         self.actiongroup_table   = gtk.ActionGroup('Table')
 
         self.actiongroup_insert.add_actions([
+            ('Duplicate', gtk.STOCK_SAVE_AS, _('Duplicate'), None,
+             _('Create a new row as a duplicate of this one'), self.record_save_new_cb),
             ('New', gtk.STOCK_NEW,   None, '', None, self.record_new_cb),
+            ('NewChild', gtk.STOCK_NEW,   _('New child row'), '',
+             _("Create e new row as child of this one"), self.record_new_child_cb),
             ])
         self.actiongroup_delete.add_actions([
             ('Delete', gtk.STOCK_DELETE, None, '<Control>k', None, self.record_delete),
@@ -264,6 +291,11 @@ class SqlTable(sqlwidget.SqlWidget):
             ('Records', None, 'Records'),
             ('MaskView', gtk.STOCK_ZOOM_IN, _('View this record in a Mask'), '<Control>m', None, self.record_in_mask),
             ('MaskViewFKey', gtk.STOCK_LEAVE_FULLSCREEN, _('View this ForeignKey in a Mask'), None, None, self.fkey_record_in_mask),
+            ('UploadImage', None, _('Upload Image'), None, None, self.upload_image),
+            ('UploadFile', None, _('Upload File'), None, None, self.upload_file),
+            ('DeleteFile', None, _('Delete File/Image'), None, None, self.delete_file),
+            ('ShowFile', None, _('Show File/Image'), None, None, self.show_file),
+            ('SaveFile', None, _('Save File/Image'), None, _('Save file locally'), self.save_file),
             ])
         self.actiongroup_table.add_actions([
             ('Export', None, _('Export'), None, None, self.export),
@@ -379,8 +411,10 @@ class SqlTable(sqlwidget.SqlWidget):
         except TypeError, e:
             obj = None
             field_name = None
+            path = None
 
         self._clicked_obj = obj  # should I officially document it?
+        self._clicked_path = path  # should I officially document it?
         self._field_name = field_name  # should I officially document it?
             
         menu = view.pop_menu(treeview, event, obj, field_name)
@@ -500,6 +534,20 @@ class SqlTable(sqlwidget.SqlWidget):
 
         return False
         
+    def key_tree_ctrl_c(self, treeview, event):
+
+        if not self.modelproxy.is_tree():
+            return
+        if not 'i' in self.mode:
+            self.sb(_("Insertion of new records is disabled. Sorry"))
+            return
+        try:
+            self.record_new(treeview, child=True)
+        except exc.ValidationError, e:
+            return True
+
+        return False
+        
     def key_tree_ctrl_s(self, treeview, event):
 
         if self.current in self.session.new and 'i' not in self.mode:
@@ -543,7 +591,7 @@ class SqlTable(sqlwidget.SqlWidget):
         if self.modelproxy.tree_field_name:
             return False
         else:
-            row = self.get_selected_row()
+            row = self.get_selected_path()
             if row is None:
                 row = -1
             else:
@@ -607,6 +655,14 @@ class SqlTable(sqlwidget.SqlWidget):
                 if not self.cell_validate():
                     return True
 
+    def record_new_child_cb(self, widget, event=None):
+        """Save the record and invokes the validator before UPDATE/INSERT a record
+        """
+        try:
+            self.record_new(parent=self._clicked_path)
+        except (exc.DialogValidationError, exc.CancelledAction):
+            return True
+    
 ###### Saving
     def set_value(self, field_name, value, path=None, fkvalue=None, initial=False, shown=False):
         """
@@ -632,13 +688,13 @@ class SqlTable(sqlwidget.SqlWidget):
         
         if not obj:
             return
-        
+
         ## casting of value
         field = self.gui_fields[field_name]
         if self.is_fkey(field_name):
-            value = field.clean_value(value, input_is_fkey=True)
+            value = field.clean_value(value, input_is_fkey=True, obj=obj)
         else:
-            value = field.clean_value(value)
+            value = field.clean_value(value, obj=obj)
         if field.editable:
             field.set_value(value, initial=initial, obj=obj, update_widget=False)
 
@@ -659,7 +715,7 @@ class SqlTable(sqlwidget.SqlWidget):
                     self.cell_entry.entry.set_text(self.gui_fields[field_name].format_value(value))
 
         if not initial:
-            self.run_hook('on_change_value', value, fkvalue, field_name=field_name)
+            self.run_hook('on_change_value', value, fkvalue, field, field_name=field_name)
 
     def get_value(self, field_name, shown=False):
         """
@@ -671,7 +727,8 @@ class SqlTable(sqlwidget.SqlWidget):
         """
         
         obj = self.get_current_obj()
-        assert obj is not None, _("no current obj: maybe no record has yet been edited")
+        if obj is None:
+            raise exc.NoCurrentObjError(_("no current obj: maybe no record has yet been edited"))
 
         if field_name == self.currently_edited_field_name:
             if self.cell_entry:
@@ -710,11 +767,41 @@ class SqlTable(sqlwidget.SqlWidget):
             return False
         return sqlwidget.SqlWidget.record_has_changed(self)
     
-    def record_save_new(self, ev):
-        ## the problem here is to prevent changes in the already used record
-        raise NotImplementedError
+    def record_save_new(self, origin=None, interactive=True):
+        """
+        Create a new record and copy all data from an origin
+
+        :param origin: the obj that must be duplicated, it can also belong
+             to a different session
+        :param interactive: (boolean) if True start editing and use set_value
+        :return: the new object
+
+        This can be used as follows in a on_save_as hook::
+
+            for row in old.righe:
+                rel_table.record_save_new(origin=row, interactive=False)
+
+        to copy all rows from one record to the other
+        
+        """
+
+        orig_obj = origin or self.get_obj_at_path(self._clicked_path)
+        path, new_obj = self.record_new(start=interactive)
+
+        for field in self.gui_fields:
+            if not field.pkey and field.persistent:
+                value = getattr(orig_obj, field.field_name)
+                if interactive:
+                    self.set_value(field.field_name, value, initial=False)
+                    # this way is alternative to editing directly, for some reason it
+                    # deletes the first edited field::
+                    #field.set_value(value, initial=False, obj=new_obj)
+                else:
+                    setattr(new_obj, field.field_name, value)
+        self.run_hook('on_save_as', orig_obj, new_obj)
+        return new_obj
     
-    def record_save(self, discard_unchanged=False, discard_new=False, ask=False):
+    def record_save(self, discard_unchanged=False, discard_new=False, ask=False, skip_check=True):
 
         """save the record
 
@@ -723,8 +810,11 @@ class SqlTable(sqlwidget.SqlWidget):
 
         see record_validate for constraint explanation
 
-        discard_empty: when record_save is called by reload, an empty record
+        :param discard_empty: when record_save is called by reload, an empty record
                        does not need validation
+        :param ask: if True a dialog box will ask if you really want to save
+        :param skip_check: (boolean) if ask=True and skip_check=True save_unsaved will skip check
+                       of real need to save
 
         """
 
@@ -733,7 +823,7 @@ class SqlTable(sqlwidget.SqlWidget):
             return
 
         if ask:
-            response = self.save_unsaved(skip_check=True, proceed=False)
+            response = self.save_unsaved(skip_check=skip_check, proceed=False)
 
             if response == gtk.RESPONSE_CANCEL:
                 raise exc.CancelledAction
@@ -798,8 +888,16 @@ class SqlTable(sqlwidget.SqlWidget):
                 self.session.expire(self.relationship_leader.current)
                 self.relationship_leader.record_display(check=False, delay_message=2)
 
-    def record_new(self, widget=None, view_name='main', start=True, obj=None, *args):
-        """prepare for 
+    def record_new(self, widget=None, view_name='main', start=True, obj=None, parent=None, *args):
+        """Add a new record
+
+        :param widget: the treeview
+        :param view_name: the name on the view
+        :param start: start editing the first non default field
+        :param obj: 
+        :param parent: if defined it's a tuple representing a path.
+                      The new row must be a child of ``parent``
+        
         """
         ## I want to avoid generating a new obj if an object was created 
         ## and not modified (as when you press Down Arrow several times
@@ -820,20 +918,22 @@ class SqlTable(sqlwidget.SqlWidget):
         ## we inhibit this 
         self.commit_inhibited = True
 
-        treeiter = self._get_next_iter(self.last_new_obj)
+        treeiter = self._get_next_iter(self.last_new_obj, parent=parent)
         path = self.modelproxy.modelstore.get_path(treeiter)
         if start:
             self.views[view_name].start_editing(path)
         ## set defaults
         if not obj:
             for field_name in self.field_list:
-                self.gui_fields[field_name].set_default()
+                self.gui_fields[field_name].set_default(obj=self.current)
 
-        self.records += [self.last_new_obj]  ## adding to records is vital to get fk correctly, in
-                                             ## case this table is a M2M of M2O
+        self.records += [self.last_new_obj]  ## adding to records is vital to get
+                                             ## fk correctly, in case 
+                                             ## this table is a M2M of M2O
         self.commit_inhibited = False
         self._new_record = True
-        return path
+        self.emit('record-new')
+        return path, self.last_new_obj
 
     def record_add(self, record):
         """
@@ -844,7 +944,7 @@ class SqlTable(sqlwidget.SqlWidget):
 #         except Exception:
 #             self.modelproxy.modelstore.append(None, [record])
         sqlwidget.SqlWidget.record_add(self, record)
-        path = self.record_new(obj=record, start=False)
+        path, obj = self.record_new(obj=record, start=False)
         self.edited_row_model_path  =  path
         self.select_path(path)
         self.record_save()
@@ -948,15 +1048,18 @@ class SqlTable(sqlwidget.SqlWidget):
         self.select_path(obj=self._clicked_obj)
         self.record_delete()
 
-    def _get_next_iter(self, new_obj):
+    def _get_next_iter(self, new_obj, parent=None):
         """
         create a new row in the next reasonable place:
 
           * the next row for simple liststores
           * the next sibling for treestores
+
+        :param new_obj: the obj to be placed there
+        :param parent: if defined it's the path of the parent row for the new object 
           
         """
-        path = self.get_selected_row()
+        path = parent or self.get_selected_path() or getattr(self, '_clicked_path', None) 
         if not path:
             try:
                 return self.modelproxy.modelstore.append([new_obj])
@@ -964,6 +1067,10 @@ class SqlTable(sqlwidget.SqlWidget):
                 return self.modelproxy.modelstore.append(None, [new_obj])
         else:
             current_iter = self.modelproxy.modelstore.get_iter(path)
+            if parent:
+                new_iter = self.modelproxy.modelstore.insert_after(current_iter, None, [new_obj])
+                self.treeview.expand_row(path, True)
+                return new_iter
             parent_iter = self.modelproxy.modelstore.iter_parent(current_iter)
             if parent_iter:
                 new_iter = self.modelproxy.modelstore.insert_after(parent_iter, current_iter, [new_obj])
@@ -977,10 +1084,11 @@ class SqlTable(sqlwidget.SqlWidget):
     
 ###### Validation
     def cell_validate(self):
-        """
-        validate cell value, return True if ok
-        The cell to be validated is the last visited cell, ie: the one whose field_name, path and cell
-        where set inside text_editing_started_cb
+        """Validate cell value and invoke field.set_value() or invoke completion.
+        Return True if ok.
+        
+        The cell to be validated is the last visited cell, ie: the one whose
+        field_name, path and cell where set inside text_editing_started_cb
         ForeignKey that do not validate should trigger completion
 
         This must be called by:
@@ -988,8 +1096,11 @@ class SqlTable(sqlwidget.SqlWidget):
           - Tab
           - Button1 on other cell
           - edited callback
-        """
 
+        IMPORTANT: validation is not a way to enforce True validation. This bypasses
+        normal completion mechanism!!!
+
+        """
         new_text = self.cell_entry.get_text()
         field_name = self.currently_edited_field_name
 
@@ -1000,6 +1111,7 @@ class SqlTable(sqlwidget.SqlWidget):
                 return True
 
             try:
+                # The next statement sets a value even if it could not pass completion!!!
                 self.set_current(filter_by={field_name :new_text})
                 return 
             except (exc.NoResultFound, exc.MultipleResultsFound), e:
@@ -1012,12 +1124,15 @@ class SqlTable(sqlwidget.SqlWidget):
                 self.sb(e)
                 return True
         else:
-            ## 
+            ##
             if self.is_fkey(field_name):
                 if self.cell_entry and self.cell_entry.is_valid():
                     return True
                 if not new_text:
-                    self.gui_fields[field_name].set_value(None, initial=False)                    
+                    # casting '' to None => you're not allowed to use '' as FK...
+                    self.set_value(field_name, None, initial=False)
+                    # self.gui_fields[field_name].set_value(None, initial=False,
+                    #                                       obj=self.current)                    
                     return True
                 self.sb(_('Value is not valid, trying completion'))
                 show = self.gui_fields[field_name].widget.completion.show_possible_completion
@@ -1025,12 +1140,20 @@ class SqlTable(sqlwidget.SqlWidget):
                 return False
             else:
                 try:
-                    self.gui_fields[field_name].set_value(new_text, initial=False)
-                    self.cell_entry.valid = True
+                    if self.cell_entry.text == new_text:
+                        return True
+                    self.set_value(field_name, new_text, initial=False)
+                    # self.gui_fields[field_name].set_value(new_text, initial=False,
+                    #                                       obj=self.current)
+                    # A hook in "on_change_value" may trigger a deletion of cell_entry
+                    # e.g. a totals.compute(), so let's check again
+                    if self.cell_entry:  
+                        self.cell_entry.text = new_text
+                        self.cell_entry.valid = True
                     return True
                 except exc.ValidationError, e:
                     self.cell_entry.valid = False
-                    self.sb(e)
+                    self.sb(str(e), seconds=8)
                     return False
         
         
@@ -1066,7 +1189,6 @@ class SqlTable(sqlwidget.SqlWidget):
         # in Masks this is done when set_field is used on record_display
         # here the display is automatic via set_cell_data_func so we have to set
         # initial value when starting editing a record
-        
         if not self.edited_row_model_path or force:
             if not obj:
                 obj = self.get_selected_obj()
@@ -1155,7 +1277,7 @@ class SqlTable(sqlwidget.SqlWidget):
         """
         from sqlkit.widgets import SqlMask, SqlTable
         try:
-            quit = self.record_save(ask=True)
+            quit = self.record_save(ask=True, skip_check=False)
         except exc.CancelledAction:
             quit = True
         if quit: 
@@ -1168,7 +1290,7 @@ class SqlTable(sqlwidget.SqlWidget):
         mask = SqlMask(self.mapper, session=self.session, layout=self.layout,
                        metadata=self.metadata, dbproxy=self.dbproxy, hooks=self.hooks)
         mask.set_mode(self.mode, reset=True)
-        mask.mode = '-b'
+        mask.mode = '-bd'
         obj = getattr(self, '_clicked_obj', self.get_selected_obj())
         try:
             del self._clicked_obj
@@ -1177,15 +1299,12 @@ class SqlTable(sqlwidget.SqlWidget):
         
         self.select_path(obj=obj)
         mask.set_records([obj])
-
-        ## let's make sure any changes in the table, after a commit, is reflected
-        ## back in the mask. This prevents a double commit with opposite meaning
-        ## check=False means that after a commit whatever is the sitution in the
-        ## mask will be disreguarded
-        def refresh_mask(table):
-            mask.record_display(check=False)
-            
-        self.connect('record-saved', refresh_mask)
+        def update_table_treeview(mask, table):
+            # emitting a 'row-changed', forces the update of the treeview but 
+            model = self.modelproxy.modelstore
+            path = table.get_selected_path()
+            model.emit('row-changed', path, model.get_iter(path))
+        mask.connect('record-saved', update_table_treeview, self)
 
         ## signals to keep selection in sync 
         def follow_selection(table, obj, mask):
@@ -1243,7 +1362,8 @@ class SqlTable(sqlwidget.SqlWidget):
 
         def disconnect(mask, id):
             self.disconnect(id)
-            del self._fmask
+            if hasattr(self, '_fmask'):
+                del self._fmask
 
         mask.connect('delete-event', disconnect, handler_id)
 
@@ -1264,12 +1384,96 @@ class SqlTable(sqlwidget.SqlWidget):
         """
         self._fk_layout_nick[field_name] = layout_nick
             
+
+    def upload_image(self, action, field_name=None):
+        """
+        Open a dialog to upoad a file and check if destination already exists.
+        Setup for a copy when record will be saved
+        """
+
+        field_name = field_name or self._field_name
+        dialog = dialogs.OpenDialog(title=_('Upload image'), default_filename='')
+        filename = dialog.filename
+        if not filename:
+            return
+        self.initialize_record_editing(self._clicked_path)
+        self.select_path(self._clicked_path)
+        save_path = self.gui_fields[field_name].get_save_path(
+                        name=dialog.new_filename or filename,
+                        obj=self.current)
+        if os.path.exists(save_path):
+            response = self.dialog(
+                type="ok-cancel",
+                text=_("A file with name '%s' already exists.\nOverwrite?") % save_path,
+                title=_('Upload name duplication'))
+            if response == gtk.RESPONSE_CANCEL:
+                return
+
+        self.gui_fields[field_name].widget.copy_file = filename # delay the real copy
+        self.gui_fields[field_name].widget.new_filename = dialog.new_filename
+        self.set_value(field_name, save_path)
+
+        return save_path
+    
+    def upload_file(self, action):
+        print action
+    
+    def delete_file(self, action, field_name=None):
+        field_name = field_name or self._field_name
+        self.initialize_record_editing(self._clicked_path)
+        self.set_value(field_name, None)
+    
+    def show_file(self, action, field_name=None):
+
+        from sqlkit.layout import image_widget
+        
+        field_name = field_name or self._field_name
+        self.initialize_record_editing(self._clicked_path, force=True, obj=self._clicked_obj)
+        filename = self.gui_fields[field_name].get_value(complete_path=True)
+        if not self.gui_fields[field_name].is_image(filename):
+            open_file(filename)
+            return
+        w = gtk.Window()
+        image = image_widget.ImageWidget()
+        image.props.width = 700
+        image.props.height = 700
+        w.add(image)
+        w.show_all()
+        
+        def change_file(table, obj):
+            if obj:
+                try:
+                    img_path = table.gui_fields[field_name].get_value(complete_path=True)
+                except exc.NoCurrentObjError, e:
+                    if table.records:
+                        table.select_path((0,))
+                        img_path = table.gui_fields[field_name].get_value(complete_path=True)
+                    else:
+                        img_path = None
+                        
+                image.set_image(img_path)
+
+        change_file(self, self.current)
+        self.connect('context-changed', change_file)
+
+    def save_file(self, action, field_name=None):
+
+        import shutil
+        
+        field_name = field_name or self._field_name
+        self.initialize_record_editing(self._clicked_path)
+        orig_filename = self.gui_fields[field_name].get_value(complete_path=True)
+        dialog = dialogs.SaveDialog(default_filename=os.path.basename(orig_filename))
+        if dialog.filename:
+            shutil.copyfile(orig_filename, dialog.filename)
+
 ###### Misc
-    def get_selected_row(self):
+    def get_selected_path(self):
         """Return the path of the selected row or None
 
-        NOTE: treeselection 'changed' signal arrives when the selection changed has
-              already occurred
+        NOTE: treeselection 'changed' signal arrives when the selection change has
+              already occurred. Use :attr:`edited_path` to get the path that
+              is being currently edited
 
         """
         selection = self.treeview.get_selection()
@@ -1301,7 +1505,7 @@ class SqlTable(sqlwidget.SqlWidget):
 
     def get_selected_obj(self):
         """Return the corrently selected obj. Return None if no selection"""
-        path = self.get_selected_row()
+        path = self.get_selected_path()
 
         if path is None:
             return None
@@ -1320,7 +1524,6 @@ class SqlTable(sqlwidget.SqlWidget):
         :param path: the modelproxy.modelstore path
         :param all: boolean: only return object of the main class (no totals)
         """
-
         obj = self.modelproxy.modelstore[path][0]
         if isinstance(obj, self.totals.total_class):
             return None
@@ -1339,12 +1542,12 @@ class SqlTable(sqlwidget.SqlWidget):
         ## row that started completion that is not ready for flushing
         ## more: it may be due for substitution!!
 
-        row = self.get_selected_row()
-        assert row is not None
+        path = self.get_selected_path()
+        assert path is not None
         
         obj = self.get_record_no_flush(pk=pk, filter_by=filter_by)
 
-        expired_obj = self.modelproxy.modelstore[row][0]
+        expired_obj = self.modelproxy.modelstore[path][0]
         if expired_obj in self.session:
             self.session.expunge(expired_obj)
         self.records.remove(expired_obj)
@@ -1352,7 +1555,7 @@ class SqlTable(sqlwidget.SqlWidget):
         ## assigning modelproxy.modelstore value will trigger 'editing-canceled'
         self.cell_entry.substitute = True
         self.cell_entry.valid = True
-        self.modelproxy.modelstore[row][0] = obj
+        self.modelproxy.modelstore[path][0] = obj
         
         if not (self.relationship_mode == 'm2m' and not self.m2m_editable):
             path = self.edited_row_model_path
@@ -1437,7 +1640,7 @@ class SqlTable(sqlwidget.SqlWidget):
         :param obj:  if path is None, the object to be searched for and selected
         """
 
-        self.views['main'].select_path(path=path, obj=obj)
+        return self.views['main'].select_path(path=path, obj=obj)
             
     def add_record(self, obj):
         """
@@ -1527,13 +1730,3 @@ class ExportDialog(dialogs.SaveDialog):
         writer.writerows(self.get_rows_func())
         f.close()
 
-
-
-
-
-
-
-
-
-
-
